"""Base class for USB connected lights.

While not intended to be instantiated directly, this class provides a
common interface for all USB connected lights and a mechanism for
discovering available lights on the system.

```python
from busylight_core import Light

all_lights = Light.all_lights()

for light in all_lights:
    light.on((255, 0, 0))  # Turn on the light with red color

for light in all_lights:
    light.off()  # Turn off all lights
````

"""

from __future__ import annotations

import abc
import contextlib
import platform
from typing import TYPE_CHECKING, ClassVar

if TYPE_CHECKING:
    from collections.abc import Callable, Generator

from functools import cache, cached_property, lru_cache

from loguru import logger

from .exceptions import (
    HardwareUnsupportedError,
    LightUnavailableError,
    NoLightsFoundError,
)
from .hardware import Hardware
from .mixins import TaskableMixin


class Light(abc.ABC, TaskableMixin):
    """Base class for USB connected lights.

    This base class provides a common interface for USB connected lights.

    Subclasses should inherit from this class, implement the abstract
    methods, and populate the `supported_device_ids` class variable
    with supported device IDs and their product names. The key for
    support_device_ids should be a tuple composed of the vendor ID and
    product ID, while the value should be the human-readable marketing
    name of the device.

    Note: If a subclass inherits from Light, but does not directly
          implement support for any specific hardware, it should leave
          the `supported_device_ids` class variable *empty*. This
          allows the base class to identify subclasses which do not
          support any specific hardware and act appropriately.

    The Light class has been designed to be helpful for discovering
    and managing USB connected lights without having to know aprori
    details of the hardware present.

    - Light.available_lights() lists devices discovered
    - Light.all_lights() returns all discovered lights ready for use
    - Light.first_light() returns the first available light

    If you know what devices you have connected and want to access
    them directly, you can use the Light subclasses directly.

    ```python
    from busylight_core.vendors.embrava import Blynclight
    from busylight_core.vendors.luxafor import Flag

    blynclight = Blynclight.first_light()
    flag = Flag.first_light()
    ```
    """

    supported_device_ids: ClassVar[dict[tuple[int, int], str]] = {}

    @classmethod
    @lru_cache(maxsize=1)
    def vendor(cls) -> str:
        """Return the vendor name in title case."""
        # EJO this is a low-effort way to get the vendor
        #     name from the module name. Subclasses can
        #     and should override this method to provide
        #     a more accurate vendor name.
        return cls.__module__.split(".")[-2].title()

    @classmethod
    @lru_cache(maxsize=1)
    def unique_device_names(cls) -> list[str]:
        """Return a list of unique device names."""
        return sorted(set(cls.supported_device_ids.values()))

    @classmethod
    @lru_cache(maxsize=1)
    def unique_device_ids(cls) -> list[tuple[int, int]]:
        """Return a list of unique device IDs."""
        return sorted(set(cls.supported_device_ids.keys()))

    @classmethod
    def claims(cls, hardware: Hardware) -> bool:
        """Return True if the hardware is claimed by this class."""
        return hardware.device_id in cls.supported_device_ids

    @classmethod
    @cache
    def subclasses(cls) -> list[type[Light]]:
        """Return a list of all subclasses of this class."""
        subclasses = []

        if cls != Light and cls.supported_device_ids:
            subclasses.append(cls)

        for subclass in cls.__subclasses__():
            subclasses.extend(subclass.subclasses())

        return sorted(subclasses, key=lambda s: s.__module__)

    @classmethod
    @lru_cache(maxsize=1)
    def supported_lights(cls) -> dict[str, list[str]]:
        """Return a dictionary of supported lights by vendor.

        Keys are vendor names, values are a list of product names.
        """
        supported_lights: dict[str, list[str]] = {}

        for subclass in cls.subclasses():
            names = supported_lights.setdefault(subclass.vendor(), [])
            names.extend(subclass.unique_device_names())

        return supported_lights

    @classmethod
    def available_lights(cls) -> dict[type[Light], list[Hardware]]:
        """Return a dictionary of available hardware by type.

        Keys are Light subclasses, values are a list of Hardware instances.

        The Hardware instances are a record of light devices that were
        discovered during the enumeration process and were claimed by
        a Light subclass. The hardware device may be in use by another
        process, which will be reported when attempting to acquire the
        device during Light subclass initialization.
        """
        available_lights: dict[type[Light], list[Hardware]] = {}

        for hardware in Hardware.enumerate():
            if cls.supported_device_ids:
                if cls.claims(hardware):
                    logger.debug(f"{cls.__name__} claims {hardware}")
                    claimed = available_lights.setdefault(cls, [])
                    claimed.append(hardware)
            else:
                for subclass in cls.subclasses():
                    if subclass.claims(hardware):
                        logger.debug(f"{subclass.__name__} claims {hardware}")
                        claimed = available_lights.setdefault(subclass, [])
                        claimed.append(hardware)

        return available_lights

    @classmethod
    def all_lights(cls, *, reset: bool = True, exclusive: bool = True) -> list[Light]:
        """Return a list of all lights ready for use.

        All the lights in the list have been initialized with the
        given reset and exclusive parameters. Lights acquired with
        exclusive=True can only be used by the current process and
        will block other processes from using the same hardware until
        the light is released.

        If no lights are found, an empty list is returned.

        :param: reset - bool - reset the hardware to a known state
        :param: exclusive - bool - acquire exclusive access to the hardware
        """
        lights: list[Light] = []

        for subclass, devices in cls.available_lights().items():
            for device in devices:
                try:
                    lights.append(subclass(device, reset=reset, exclusive=exclusive))
                except Exception as error:
                    logger.info(f"Failed to acquire {device}: {error}")

        return lights

    @classmethod
    def first_light(cls, *, reset: bool = True, exclusive: bool = True) -> Light:
        """Return the first unused light ready for use.

        The light returned has been initialized with the given reset
        and exclusive parameters. If exclusive=True, the light can
        only be used by the current process and will block other
        processes from using the light until it is released.

        Raises:
        - NoLightsFoundError

        """
        for subclass, devices in cls.available_lights().items():
            for device in devices:
                try:
                    return subclass(device, reset=reset, exclusive=exclusive)
                except Exception as error:
                    logger.info(f"Failed to acquire {device}: {error}")
                    raise

        raise NoLightsFoundError(cls)

    @classmethod
    def udev_rules(cls, mode: int = 0o666) -> dict[tuple[int, int], list[str]]:
        """Return a dictionary of udev rules for the light subclass.

        The keys of the dictionary are device ID tuples, while the
        values are lists of udev rules for a particular light.  If
        duplicate device IDs are encountered, the first device ID
        wins and subsequent device IDs are ignored.

        :param mode: int - file permissions for the device, defaults to 0o666
        """
        rules = {}

        rule_formats = [
            'SUBSYSTEMS=="usb", ATTRS{{idVendor}}=="{vid:04x}", ATTRS{{idProduct}}=="{pid:04x}", MODE="{mode:04o}"',  # noqa: E501
            'KERNEL=="hidraw*", ATTRS{{idVendor}}=="{vid:04x}", ATTRS{{idProduct}}=="{pid:04x}", MODE="{mode:04o}"',  # noqa: E501
        ]

        if cls.supported_device_ids:
            for vid, pid in cls.unique_device_ids():
                content = rules.setdefault((vid, pid), [])
                content.append(f"# {cls.vendor()} {cls.__name__} udev rules")
                for rule_format in rule_formats:
                    content.append(rule_format.format(vid=vid, pid=pid, mode=mode))
        else:
            for subclass in cls.subclasses():
                subclass_rules = subclass.udev_rules(mode=mode)
                for key, value in subclass_rules.items():
                    if key not in rules:
                        rules[key] = value

        return rules

    def __init__(
        self,
        hardware: Hardware,
        *,
        reset: bool = False,
        exclusive: bool = True,
    ) -> None:
        """Initialize a Light with the given hardware information.

        The hardware argument is an instance of the Hardware class
        usually obtained from the Hardware.enumerate method. Due to
        vagaries in vendor USB implementations, the supplied hardware
        can contain incomplete information compared to other vendors
        and it's up to the Light subclass to fill in some of the
        blanks.

        The reset keyword-only parameter controls whether the hardware
        is reset to a known state using the Light.reset method.

        The exclusive keyword-only parameter controls whether the
        process creating this light has exclusive access to the
        hardware.

        :param: hardware  - Hardware
        :param: reset     - bool
        :param: exclusive - bool

        Raises:
        - HardwareUnsupportedError

        """
        if not self.__class__.claims(hardware):
            raise HardwareUnsupportedError(hardware, self.__class__)

        self.hardware = hardware
        self._reset = reset
        self._exclusive = exclusive

        if exclusive:
            self.hardware.acquire()

        if reset:
            self.reset()

    def __repr__(self) -> str:
        repr_fmt = "{n}({h!r}, *, reset={r}, exclusive={e}"
        return repr_fmt.format(
            n=self.__class__.__name__,
            h=self.hardware,
            r=self._reset,
            e=self._exclusive,
        )

    @cached_property
    def path(self) -> str:
        """The path to the hardware device."""
        return self.hardware.path.decode("utf-8")

    @cached_property
    def platform(self) -> str:
        """The discovered operating system platform name."""
        system = platform.system()
        match system:
            case "Windows":
                return f"{system}_{platform.release()}"
            case _:
                return system

    @property
    def exclusive(self) -> bool:
        """Return True if the light has exclusive access to the hardware."""
        return self._exclusive

    @property
    def was_reset(self) -> bool:
        """Return True if the light was reset when the hardware was initialized."""
        return self._reset

    @cached_property
    def sort_key(self) -> tuple[str, str, str]:
        """Return a tuple used for sorting lights.

        The tuple consists of:
        - vendor name in lowercase
        - device name in lowercase
        - hardware path
        """
        return (self.vendor().lower(), self.name.lower(), self.path)

    def __eq__(self, other: object) -> bool:
        try:
            return self.sort_key == other.sort_key
        except AttributeError:
            raise TypeError from None

    def __lt__(self, other: Light) -> bool:
        if not isinstance(other, Light):
            return NotImplemented

        for self_value, other_value in zip(self.sort_key, other.sort_key, strict=False):
            if self_value != other_value:
                return self_value < other_value

        return False

    def __hash__(self) -> int:
        """Return a hash value for the light based on its sort key."""
        try:
            return self._hash
        except AttributeError:
            self._hash = hash(self.sort_key)
            return self._hash

    @cached_property
    def name(self) -> str:
        """The human-readable marketing name of this light."""
        return self.supported_device_ids[self.hardware.device_id]

    @property
    def hex(self) -> str:
        """The hexadecimal representation of the light's state."""
        return bytes(self).hex(":")

    @property
    def read_strategy(self) -> Callable[[int, int | None], bytes]:
        """The read method used by this light."""
        return self.hardware.handle.read

    @property
    def write_strategy(self) -> Callable[[bytes], None]:
        """The write method used by this light."""
        return self.hardware.handle.write

    @contextlib.contextmanager
    def exclusive_access(self) -> Generator[None, None, None]:
        """Manage exclusive access to the light.

        If the device is not acquired in exclusive mode, it will be
        acquired and released automatically.

        No actions are taken if the light is already acquired
        in exclusive mode.
        """
        if not self._exclusive:
            self.hardware.acquire()

        yield

        if not self._exclusive:
            self.hardware.release()

    def update(self) -> None:
        """Obtain the current state of the light and write it to the device.

        Raises:
        - LightUnavailableError

        """
        state = bytes(self)

        match self.platform:
            case "Windows_10":
                state = bytes([0]) + state
            case "Darwin" | "Linux" | "Windows_11":
                pass
            case _:
                logger.info(f"Unsupported OS {self.platform}, hoping for the best.")

        with self.exclusive_access():
            logger.debug(f"{self.name} payload {state.hex(':')}")
            try:
                self.write_strategy(state)
            except Exception as error:
                logger.error(f"{self}: {error}")
                raise LightUnavailableError(self) from None

    @contextlib.contextmanager
    def batch_update(self) -> Generator[None, None, None]:
        """Update the software state of the light on exit.

        This context manager is convenience for updating multiple
        light attribute values at once and write the new state to the
        hardware on exit. This approach reduces the number of writes
        to the hardware, which is beneficial for performance.
        """
        yield
        self.update()

    @abc.abstractmethod
    def on(
        self,
        color: tuple[int, int, int],
        led: int = 0,
    ) -> None:
        """Activate the light with the given red, green, blue color tuple.

        If a subclass supports multiple LEDs, the `led` parameter
        specifies which LED to activate. If `led` is 0, all LEDs
        are activated with the specified color. If a subclasss
        does not support multiple LEDs, the `led` parameter
        is ignored and defaults to 0.

        Color tuple values should be in the range of 0-255.

        :param: color: tuple[int, int, int] - RGB color tuple
        :param: led: int - LED index, 0 for all LEDs
        """
        raise NotImplementedError

    def off(self, led: int = 0) -> None:
        """Deactivate the light.

        If a subclass supports multiple LEDs, the `led` parameter
        specifies which LED to deactivate. If `led` is 0, all LEDs
        are deactivated. If a subclass does not support multiple LEDs,
        the `led` parameter is ignored and defaults to 0.

        :param: led: int - LED index, 0 for all LEDs
        """
        self.on((0, 0, 0), led)

    def reset(self) -> None:
        """Turn the light off and cancel associated asynchronous tasks."""
        self.off()
        self.cancel_tasks()

    @abc.abstractmethod
    def __bytes__(self) -> bytes:
        """Return the light's state suitable for writing to the device."""
        raise NotImplementedError

    @property
    @abc.abstractmethod
    def color(self) -> tuple[int, int, int]:
        """Get the current RGB color of the light."""
        raise NotImplementedError

    @color.setter
    @abc.abstractmethod
    def color(self, value: tuple[int, int, int]) -> None:
        """Set the RGB color of the light.

        :param: value: tuple[int, int, int] - RGB color tuple
        """
        raise NotImplementedError

    @property
    def is_lit(self) -> bool:
        """Check if the light is currently lit."""
        return any(self.color)



"""Color manipulation mixin for Light classes."""


class ColorableMixin:
    """Mixin providing color properties."""

    red = property(
        lambda self: getattr(self, "_red", 0),
        lambda self, value: setattr(self, "_red", value),
        doc="Red color value.",
    )

    green = property(
        lambda self: getattr(self, "_green", 0),
        lambda self, value: setattr(self, "_green", value),
        doc="Green color value.",
    )

    blue = property(
        lambda self: getattr(self, "_blue", 0),
        lambda self, value: setattr(self, "_blue", value),
        doc="Blue color value.",
    )

    @property
    def color(self) -> tuple[int, int, int]:
        """Tuple of RGB color values."""
        return (self.red, self.green, self.blue)

    @color.setter
    def color(self, value: tuple[int, int, int]) -> None:
        self.red, self.green, self.blue = value



"""Luxafor vendor base class."""

from loguru import logger

from busylight_core.hardware import Hardware
from busylight_core.light import Light


class LuxaforBase(Light):
    """Base class for Luxafor devices.

    Provides common functionality for all Luxafor devices including
    the Flag, Mute, Orb, Bluetooth, and BusyTag product lines.
    """

    @staticmethod
    def vendor() -> str:
        """Return the vendor name for Luxafor devices."""
        return "Luxafor"

    @classmethod
    def claims(cls, hardware: Hardware, product_check: bool = True) -> bool:
        """Return True if the hardware matches Luxafor criteria.

        The product_check argument will short-circuit checking
        the hardware product string for Luxafor device names.
        This is used primarily by the busytag implementation
        which does not require the product string check.

        :param hardware: The Hardware instance to check.
        :product_check: Whether to check the product string.
        """
        result = super().claims(hardware)

        if not product_check:
            return result

        if not result:
            return False

        try:
            product = hardware.product_string.split()[-1].casefold()
        except (KeyError, IndexError) as error:
            logger.debug(f"problem {error} processing {hardware}")
            return False

        return product in [
            value.casefold() for value in cls.supported_device_ids.values()
        ]



"""ThingM blink(1) Support"""

from collections.abc import Callable
from functools import cached_property
from typing import ClassVar

from ._blink1 import LEDS, Action, Report, State
from .thingm_base import ThingMBase


class Blink1(ThingMBase):
    """ThingM Blink(1) status light controller.

    The Blink(1) is a USB-connected RGB LED device that uses
    feature reports for communication and supports various effects.
    """

    supported_device_ids: ClassVar[dict[tuple[int, int], str]] = {
        (0x27B8, 0x01ED): "Blink(1)",
    }

    @cached_property
    def state(self) -> State:
        """The device state manager."""
        return State()

    def __bytes__(self) -> bytes:
        return bytes(self.state)

    def on(self, color: tuple[int, int, int], led: int = 0) -> None:
        """Turn on the Blink(1) with the specified color.

        :param color: RGB color tuple (red, green, blue) with values 0-255
        :param led: LED index (0 for the first LED, 1 for the second, etc.)
        """
        with self.batch_update():
            self.state.clear()
            self.state.report = Report.One
            self.state.action = Action.FadeColor
            self.state.color = color
            self.state.fade = 10  # Default fade time in milliseconds
            self.state.leds = LEDS(led)

    @property
    def color(self) -> tuple[int, int, int]:
        """Tuple of RGB color values."""
        return self.state.color

    @color.setter
    def color(self, value: tuple[int, int, int]) -> None:
        self.state.color = value

    @property
    def write_strategy(self) -> Callable:
        """The write strategy for communicating with the device."""
        return self.hardware.handle.send_feature_report



"""Plantronics vendor base class."""

from busylight_core.vendors.embrava.embrava_base import EmbravaBase


class PlantronicsBase(EmbravaBase):
    """Base class for Plantronics devices.

    Plantronics devices are typically OEM versions of Embrava devices
    with the same functionality but different vendor branding.
    """

    @staticmethod
    def vendor() -> str:
        """Return the vendor name for Plantronics devices."""
        return "Plantronics"



"""Agile Innovative vendor base class."""

from busylight_core.light import Light


class AgileInnovativeBase(Light):
    """Base class for Agile Innovative devices.

    Provides common functionality for all Agile Innovative devices,
    primarily the BlinkStick product line.
    """

    @staticmethod
    def vendor() -> str:
        """Return the vendor name for Agile Innovative devices."""
        return "Agile Innovative"



"""USB Hardware Description

This module provides a description of USB hardware devices, including
HID and serial connections.

"""

from __future__ import annotations

import contextlib
from dataclasses import dataclass
from enum import Enum
from functools import cached_property
from typing import TYPE_CHECKING

import serial
from loguru import logger
from serial.tools import list_ports

if TYPE_CHECKING:
    from serial.tools.list_ports_common import ListPortInfo

from . import hid
from .exceptions import InvalidHardwareError


class ConnectionType(int, Enum):
    """USB Hardware connection types."""

    ANY: int = -1
    UNKNOWN: int = 0
    HID: int = 1
    SERIAL: int = 2
    BLUETOOTH: int = 3


HardwareHandle = hid.Device | serial.Serial


@dataclass
class Hardware:
    """USB Hardware description."""

    device_type: ConnectionType
    path: bytes
    vendor_id: int
    product_id: int
    serial_number: str
    manufacturer_string: str
    product_string: str | None = None
    release_number: str | None = None
    usage: int | None = None
    usage_page: int | None = None
    interface_number: int | None = None
    bus_type: int | None = None
    is_acquired: bool = False

    @classmethod
    def enumerate(cls, by_type: ConnectionType = ConnectionType.ANY) -> list[Hardware]:
        """List of all connected hardware devices.

        Raises:
        - NotImplementedError

        """
        hardware_info = []

        match by_type:
            case ConnectionType.ANY:
                for connection_type in list(ConnectionType)[2:]:
                    with contextlib.suppress(NotImplementedError):
                        hardware_info.extend(cls.enumerate(connection_type))

            case ConnectionType.HID:
                for device_dict in hid.enumerate():
                    with contextlib.suppress(InvalidHardwareError):
                        hardware_info.append(cls.from_hid(device_dict))

            case ConnectionType.SERIAL:
                for port_info in list_ports.comports():
                    with contextlib.suppress(InvalidHardwareError):
                        hardware_info.append(cls.from_portinfo(port_info))

            case _:
                msg = f"Device connection {by_type.name} not implemented"
                raise NotImplementedError(msg)

        return hardware_info

    @classmethod
    def from_portinfo(cls, port_info: ListPortInfo) -> Hardware:
        """Create a Hardware object from a serial port info object.

        Raises:
        - InvalidHardwareError

        """
        try:
            return cls(
                device_type=ConnectionType.SERIAL,
                vendor_id=port_info.vid,
                product_id=port_info.pid,
                path=port_info.device.encode("utf-8"),
                serial_number=port_info.serial_number,
                manufacturer_string=port_info.manufacturer,
                product_string=port_info.product,
                bus_type=1,
            )
        except Exception:
            logger.exception("%s", port_info)
            raise InvalidHardwareError(port_info.__dict__) from None

    @classmethod
    def from_hid(cls, device: dict) -> Hardware:
        """Create a Hardware object from a HID dictionary.

        Raises:
        - InvalidHardwareError

        """
        try:
            return cls(device_type=ConnectionType.HID, **device)
        except Exception:
            logger.exception("%s", device)
            raise InvalidHardwareError(device) from None

    @cached_property
    def device_id(self) -> tuple[int, int]:
        """A tuple of the vendor and product identifiers.

        Each item in the tuple is a 16-bit integer.
        """
        return (self.vendor_id, self.product_id)

    def __post_init__(self) -> None:
        self.vendor_id &= 0x0FFFF
        self.product_id &= 0x0FFFF

    def __str__(self) -> str:
        return "{vid:04x}:{pid:04x} {man} {prod} {ser} {path}".format(
            vid=self.vendor_id,
            pid=self.product_id,
            man=self.manufacturer_string,
            prod=self.product_string or "",
            ser=self.serial_number or "",
            path=self.path.decode("utf-8"),
        )

    @cached_property
    def handle(self) -> HardwareHandle:
        """Hardware device I/O handle."""
        handle: HardwareHandle

        match self.device_type:
            case ConnectionType.HID:
                handle = hid.Device()
            case ConnectionType.SERIAL:
                handle = serial.Serial(timeout=1)
                handle.port = self.path.decode("utf-8")
            case _:
                msg = f"Device type {self.device_type} not implemented"
                raise NotImplementedError(msg)
        return handle

    def acquire(self) -> None:
        """Open the hardware device.

        The device is available for I/O operations after this method
        returns. If the device is already acquired, no actions are
        taken.

        """
        if self.is_acquired:
            logger.debug(f"{self} already acquired")
            return

        match self.device_type:
            case ConnectionType.HID:
                self.handle.open_path(self.path)
                self.is_acquired = True
            case ConnectionType.SERIAL:
                self.handle.open()
                self.is_acquired = True
            case _:
                msg = f"{self.device_type.value.title()} hardware not implemented"
                raise NotImplementedError(msg)

    def release(self) -> None:
        """Close the hardware device.

        Subsequent I/O operations to this device will fail after this
        method returns. If the device has already been released, no
        action is taken.
        """
        if not self.is_acquired:
            logger.debug(f"{self} already released")
            return

        match self.device_type:
            case ConnectionType.HID | ConnectionType.SERIAL:
                self.handle.close()
                self.is_acquired = False
            case _:
                msg = f"{self.device_type.value.title()} hardware not implemented"
                raise NotImplementedError(msg)



"""CompuLab vendor base class."""

from busylight_core.light import Light


class CompuLabBase(Light):
    """Base class for CompuLab devices.

    Provides common functionality for all CompuLab devices,
    primarily the fit-statUSB product line.
    """

    @staticmethod
    def vendor() -> str:
        """Return the vendor name for CompuLab devices."""
        return "CompuLab"



"""ThingM vendor base class."""

from busylight_core.light import Light


class ThingMBase(Light):
    """Base class for ThingM devices.

    Provides common functionality for all ThingM devices,
    primarily the Blink(1) product line.
    """

    @staticmethod
    def vendor() -> str:
        """Return the vendor name for ThingM devices."""
        return "ThingM"



"""Kuando vendor base class."""

from busylight_core.light import Light


class KuandoBase(Light):
    """Base class for Kuando devices.

    Provides common functionality for all Kuando devices,
    primarily the Busylight product line.
    """

    @staticmethod
    def vendor() -> str:
        """Return the vendor name for Kuando devices."""
        return "Kuando"



"""Embrava Blynclight Support"""

from typing import ClassVar

from .embrava_base import EmbravaBase


class Blynclight(EmbravaBase):
    """Embrava Blynclight status light controller.

    The Blynclight is a USB-connected RGB LED device with additional features
    like sound playback, volume control, and flashing patterns.
    """

    supported_device_ids: ClassVar[dict[tuple[int, int], str]] = {
        (0x2C0D, 0x0001): "Blynclight",
        (0x2C0D, 0x000C): "Blynclight",
        (0x0E53, 0x2516): "Blynclight",
    }



"""Agile Innovative BlinkStick"""

from functools import cached_property
from typing import ClassVar

from busylight_core import Hardware

from ._blinkstick import State
from .blinkstick_base import BlinkStickBase


class BlinkStick(BlinkStickBase):
    """Agile Innovative BlinkStick status light controller.

    The BlinkStick is a USB-connected RGB LED device that can be controlled
    to display various colors and patterns for status indication.
    """

    supported_device_ids: ClassVar[dict[tuple[int, int], str]] = {
        (0x20A0, 0x41E5): "BlinkStick",
    }

    @classmethod
    def claims(cls, hardware: Hardware) -> bool:
        """Return True if the hardware matches a BlinkStick."""
        if not super().claims(hardware):
            return False
        try:
            major, _ = cls.get_version(hardware.serial_number)
        except ValueError:
            return False
        return major == 1

    @cached_property
    def state(self) -> State:
        """Get the current state of the BlinkStick."""
        return State.blinkstick()
